/****************************************************************/
/*                                                              */
/*      basicdlg.cpp                                            */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "basicdlg.h"
#include "utils.h"
#include <kapp.h>
#include <kmsgbox.h>
#include <kcolorbtn.h>
#include <qgroupbox.h>
#include <qlineedit.h>

BasicDlg::BasicDlg(QWidget *parent) : QDialog(parent, 0, TRUE)
{
  setCaption(i18n("Basic settings"));
  g1 = new QGroupBox(i18n("Format"), this);
  g1->setGeometry(12, 6, 274, 112);
  l1 = new QLabel(i18n("Width"), this);
  l1->setGeometry(22, 36, 46, 22);
  formatx = new QLineEdit(this);
  formatx->setGeometry(68, 36, 50, 22);
  l2 = new QLabel("cm", this);
  l2->setGeometry(122, 36, 30, 22);
  l3 = new QLabel(i18n("Height"), this);
  l3->setGeometry(156, 36, 44, 22);
  formaty = new QLineEdit(this);
  formaty->setGeometry(200, 36, 50, 22);
  l4 = new QLabel("cm", this);
  l4->setGeometry(254, 36, 30, 22);
  portrait = new QPushButton(i18n("Portrait A4"), this);
  portrait->setGeometry(28, 76, 110, 25);
  connect(portrait, SIGNAL(clicked()), SLOT(slotPortrait()));
  landscape = new QPushButton(i18n("Landscape A4"), this);
  landscape->setGeometry(158, 76, 110, 25);
  connect(landscape, SIGNAL(clicked()), SLOT(slotLandscape()));
  g2 = new QGroupBox(i18n("Default colors"), this);
  g2->setGeometry(12, 124, 274, 130);
  l5 = new QLabel(i18n("Frame"), this);
  l5->setGeometry(22, 149, 86, 25);
  colFrame = new KColorButton(this, 0);
  colFrame->setGeometry(108, 149, 80, 25);
  l6 = new QLabel(i18n("Grid"), this);
  l6->setGeometry(22, 181, 86, 25);
  colGrid = new KColorButton(this, 0);
  colGrid->setGeometry(108, 181, 80, 25);
  l7 = new QLabel(i18n("Data"), this);
  l7->setGeometry(22, 213, 86, 25);
  colData = new KColorButton(this, 0);
  colData->setGeometry(108, 213, 80, 25);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(12, 266, 80, 25);
  ok->setDefault(TRUE);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(102, 266, 80, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(204, 266, 80, 25);
  connect(help, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

BasicDlg::~BasicDlg()
{
  delete g1;
  delete g2;
  delete l1;
  delete l2;
  delete l3;
  delete l4;
  delete l5;
  delete l6;
  delete l7;
  delete formatx;
  delete formaty;
  delete portrait;
  delete landscape;
  delete ok;
  delete cancel;
  delete colFrame;
  delete colGrid;
  delete colData;
  delete help;
}

void BasicDlg::setWidgets(basicStruct bs)
{
  QString s;

  formatx->setText(s.setNum(bs.xf, 'f', 2));
  formaty->setText(s.setNum(bs.yf, 'f', 2));
  colFrame->setColor(QColor(bs.colFrame));
  colGrid->setColor(QColor(bs.colGrid));
  colData->setColor(QColor(bs.colData));
}

basicStruct BasicDlg::getValues()
{
  b.colFrame = colFrame->color().rgb();
  b.colGrid = colGrid->color().rgb();
  b.colData = colData->color().rgb();
  return b;
}

void BasicDlg::slotPortrait()
{
  formatx->setText("21.00");
  formaty->setText("29.70");
}

void BasicDlg::slotLandscape()
{
  formatx->setText("29.70");
  formaty->setText("21.00");
}

void BasicDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-basic");
}

void BasicDlg::slotOK()
{
  if (!getVal(this, formatx, &b.xf, i18n("invalid width!")))
    return;
  if (!getVal(this, formaty, &b.yf, i18n("invalid height!")))
    return;
  accept();
}

