/****************************************************************/
/*                                                              */
/*      funcdlg.cpp                                             */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "kpl.h"
#include "funcdlg.h"
#include "symboldlg.h"
#include "utils.h"
#include <kapp.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <kcolorbtn.h>
#include <qfileinfo.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qgroupbox.h>
#include <dlfcn.h>

void getPar(QWidget *parent, double *p)
{
  int i;
  char s[81];
  FILE *f;

  QString filebuf = KFileDialog::getOpenFileName(0, "*.par");
  if (!filebuf.isEmpty())
    if ((f = fopen(filebuf, "r"))) {
      i = -1;
      while (fgets(s, 80, f) && (++i < 20))
        if (sscanf(s, "%lg", &p[i]) != 1)
          break;
      fclose(f);
    } else
      KMsgBox::message(parent, i18n("Error"), i18n("while trying to open parameter file"), 2);
}

ChooseFuncDlg::ChooseFuncDlg(QWidget *parent, FILE *f, QLineEdit *func0) :
 QDialog(parent, 0, true)
{
  char s[256];

  func = func0;
  setCaption(i18n("Functions"));
  funcList = new QListBox(this);
  funcList->setGeometry(10, 12, 208, 238);
  while ((fgets(s, 256, f)))
    if (s[9] == 'T')
      funcList->insertItem(strtok(&s[11], "\n"));
  fclose(f);
  connect(funcList, SIGNAL(selected(int)), SLOT(slotList(int)));
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(8, 264, 100, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(120, 264, 100, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  adjustSize();
  setFixedSize(size());
}

ChooseFuncDlg::~ChooseFuncDlg()
{
  delete funcList;
  delete ok;
  delete cancel;
}

void ChooseFuncDlg::slotList(int i)
{
  func->setText(funcList->text(i));
  accept();
}

void ChooseFuncDlg::slotOK()
{
  int i;

  if ((i = funcList->currentItem()) != -1) {
    func->setText(funcList->text(i));
    accept();
  }
}

EditParameterDlg::EditParameterDlg(QWidget *parent, double *p0) :
 QDialog(parent, 0, TRUE)
{
  QString s;
  int i, x, y;

  p = p0;
  setCaption(i18n("Parameters"));
  for (i = 0; i < 20; i++) {
    x = 128 * (i / 10);
    y = 32 * (i % 10);
    lp[i] = new QLabel("p", this);
    lp[i]->setGeometry(x + 10, y + 14, 8, 22);
    ln[i] = new QLabel(s.setNum(i), this);
    ln[i]->setGeometry(x + 18, y + 20, 20, 22);
    par[i] = new QLineEdit(this);
    par[i]->setGeometry(x + 40, y + 14, 70, 22);
    par[i]->setText(s.setNum(p[i]));
  }
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(18, 338, 100, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(142, 338, 100, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  adjustSize();
  setFixedSize(size());
}

EditParameterDlg::~EditParameterDlg()
{
  int i;

  for (i = 0; i < 20; i++) {
    delete lp[i];
    delete ln[i];
    delete par[i];
  }
  delete ok;
  delete cancel;
}

void EditParameterDlg::slotOK()
{
  bool valid = true;
  QString s;
  int i;
  double pt[20];

  for (i = 0; i < 20; i++) {
    s = par[i]->text();
    pt[i] = s.toDouble(&valid);
    if (!valid) {
      KMsgBox::message(this, i18n("Error"),
                       s.sprintf(s, i18n("invalid parameter %i"), i), 2);
      return;
    }
  }
  memcpy(p, pt, 20 * sizeof(double));
  accept();
}

FuncDlg::FuncDlg(QWidget *parent, KPl *kpa, PFUNDATA fd0) :
 QDialog(parent, 0, TRUE)
{
  QString s;

  kp = kpa;
  fd = fd0;
  fdt = new FUNDATA;
  *fdt = *fd;
  setCaption(i18n("Function"));
  g1 = new QGroupBox("y(x)", this);
  g1->setGeometry(10, 4, 370, 144);
  l1 = new QLabel(i18n("Library"), this);
  l1->setGeometry(20, 30, 90, 22);
  fileName = new QLineEdit(this);
  fileName->setGeometry(110, 30, 225, 22);
  fileName->setText(fd->path);
  chFile = new QPushButton("...", this);
  chFile->setGeometry(345, 29, 25, 25);
  connect(chFile, SIGNAL(clicked()), SLOT(slotFile()));
  l2 = new QLabel(i18n("Function"), this);
  l2->setGeometry(20, 70, 90, 22);
  func = new QLineEdit(this);
  func->setGeometry(110, 70, 225, 22);
  func->setText(fd->name);
  chFunc = new QPushButton("...", this);
  chFunc->setGeometry(345, 69, 25, 25);
  connect(chFunc, SIGNAL(clicked()), SLOT(slotFunc()));
  l13 = new QLabel(i18n("Parameter"), this);
  l13->setGeometry(20, 110, 90, 22);
  par = new QPushButton(i18n("Edit"), this);
  par->setGeometry(110, 109, 80, 25);
  connect(par, SIGNAL(clicked()), SLOT(slotEditParameter()));
  chPar = new QPushButton(i18n("Load"), this);
  chPar->setGeometry(200, 109, 80, 25);
  connect(chPar, SIGNAL(clicked()), SLOT(slotGetParameter()));
  g2 = new QGroupBox(this);
  g2->setGeometry(10, 154, 370, 176);
  l3 = new QLabel("x", this);
  l3->setGeometry(20, 174, 10, 22);
  l4 = new QLabel("min", this);
  l4->setGeometry(28, 178, 28, 22);
  exMin = new QLineEdit(this);
  exMin->setGeometry(110, 174, 50, 22);
  exMin->setText(s.setNum(fd->xfmin));
  l5 = new QLabel("x", this);
  l5->setGeometry(230, 174, 10, 22);
  l6 = new QLabel("max", this);
  l6->setGeometry(238, 178, 28, 22);
  exMax = new QLineEdit(this);
  exMax->setGeometry(320, 174, 50, 22);
  exMax->setText(s.setNum(fd->xfmax));
  l7 = new QLabel("D", this);
  l7->setGeometry(20, 215, 10, 22);
  QFont f("Symbol");
  l7->setFont(f);
  l8 = new QLabel("x", this);
  l8->setGeometry(30, 214, 54, 22);
  exStep = new QLineEdit(this);
  exStep->setGeometry(110, 214, 50, 22);
  exStep->setText(s.setNum(fd->dx));
  l9 = new QLabel(i18n("x normalization"), this);
  l9->setGeometry(20, 254, 90, 22);
  efx = new QLineEdit(this);
  efx->setGeometry(110, 254, 50, 22);
  efx->setText(s.setNum(fd->fx));
  l10 = new QLabel(i18n("y normalization"), this);
  l10->setGeometry(230, 254, 90, 22);
  efy = new QLineEdit(this);
  efy->setGeometry(320, 254, 50, 22);
  efy->setText(s.setNum(fd->fy));
  l11 = new QLabel(i18n("Symbol"), this);
  l11->setGeometry(20, 296, 62, 22);
  symb = new QSpinBox(-17, 9, 1, this);
  symb->setGeometry(110, 296, 50, 22);
  symb->setValue(fd->symb);
  chSymb = new QPushButton("...", this);
  chSymb->setGeometry(170, 295, 25, 25);
  connect(chSymb, SIGNAL(clicked()), SLOT(slotSymbol()));
  l12 = new QLabel(i18n("Color"), this);
  l12->setGeometry(230, 296, 90, 22);
  colData = new KColorButton(this, 0);
  colData->setGeometry(320, 295, 50, 25);
  colData->setColor(fd->color);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(10, 342, 85, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  apply = new QPushButton(i18n("Apply"), this);
  apply->setGeometry(105, 342, 85, 25);
  connect(apply, SIGNAL(clicked()), SLOT(slotApply()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(200, 342, 85, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(295, 342, 85, 25);
  connect(help, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

FuncDlg::~FuncDlg()
{
  delete fdt;
  delete l1;
  delete l2;
  delete l3;
  delete l4;
  delete l5;
  delete l6;
  delete l7;
  delete l8;
  delete l9;
  delete l10;
  delete l11;
  delete l12;
  delete g1;
  delete g2;
  delete efx;
  delete efy;
  delete fileName;
  delete func;
  delete exMin;
  delete exMax;
  delete exStep;
  delete symb;
  delete colData;
  delete chFile;
  delete chFunc;
  delete chSymb;
  delete chPar;
  delete par;
  delete apply;
  delete ok;
  delete cancel;
  delete help;
}

void FuncDlg::slotFile()
{
  QString s = KFileDialog::getOpenFileName(0, "*.so");
  if (!s.isEmpty())
    fileName->setText(s);
}

void FuncDlg::slotFunc()
{
  char path[260];
  FILE *f;

  QFileInfo fi(fileName->text());
  sprintf(path, "%s/%s.def", (const char*) fi.dirPath(true),
          (const char*) fi.baseName());
  if ((f = fopen(path, "r"))) {
    ChooseFuncDlg dlg(this, f, func);
    dlg.exec();
  } else
    KMsgBox::message(this, i18n("Error"), i18n("while trying to open module definition file"), 2);
}

void FuncDlg::slotGetParameter()
{
  getPar(this, fdt->p);
}

void FuncDlg::slotEditParameter()
{
  EditParameterDlg dlg(this, fdt->p);
  dlg.exec();
}

void FuncDlg::slotSymbol()
{
  SymbolDlg dlg(this, symb);
  dlg.exec();
}

void FuncDlg::getValues(bool ok)
{
  char msg[256], *error;

  if (!getVal(this, efx, &fdt->fx, i18n("invalid fx!")))
    return;
  if (!getVal(this, efy, &fdt->fy, i18n("invalid fy!")))
    return;
  if (!getVal(this, exMin, &fdt->xfmin, i18n("invalid xmin!")))
    return;
  if (!getVal(this, exMax, &fdt->xfmax, i18n("invalid xmax!")))
    return;
  if (!getVal(this, exStep, &fdt->dx, i18n("invalid dx!")))
    return;
  if (strcmp(fd->path, fileName->text())) {
    strcpy(fdt->path, fileName->text());
    if (fdt->path[0]) {
      if (!(fdt->hmod = dlopen(fdt->path, RTLD_NOW))) {
        sprintf(msg, i18n("while trying to open\nfile %s:\n%s"),
                fdt->path, dlerror());
        KMsgBox::message(this, i18n("Error"), msg);
        return;
      }
    } else
      fdt->hmod = 0;
  }
  if (strcmp(fd->name, func->text()) || fdt->hmod) {
    strcpy(fdt->name, func->text());
    if (fdt->name[0]) {
      if (!fdt->hmod) {
        KMsgBox::message(this, i18n("Error"), i18n("while trying to open library"));
        return;
      }
      fdt->fkt = (double (*)(double, double *)) dlsym(fdt->hmod, fdt->name);
      if ((error = (char *) dlerror()) != 0) {
        sprintf(msg, i18n("while trying to determine the address of\nfunction %s:\n%s"),
                fdt->name, error);
        KMsgBox::message(this, i18n("Error"), msg);
      }
    } else
      fdt->fkt = 0;
  }
  if (fd->hmod && (fd->hmod != fdt->hmod))
    dlclose(fd->hmod);
  *fd = *fdt;
  fd->color = colData->color().rgb();
  fd->symb = symb->value();
  kp->changed = true;
  kp->updWin();
  if (ok)
    accept();
}

void FuncDlg::slotApply()
{
  getValues(false);
}

void FuncDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-func");
}

void FuncDlg::slotOK()
{
  getValues(true);
}

ParFuncDlg::ParFuncDlg(QWidget *parent, KPl *kpa, PPARFUNDATA fd0) :
 QDialog(parent, 0, TRUE)
{
  QString s;

  kp = kpa;
  fd = fd0;
  fdt = new PARFUNDATA;
  *fdt = *fd;
  setCaption(i18n("Parametric function"));
  g1 = new QGroupBox("x(t)", this);
  g1->setGeometry(10, 4, 370, 144);
  l1 = new QLabel(i18n("Library"), this);
  l1->setGeometry(20, 30, 90, 22);
  fileNamex = new QLineEdit(this);
  fileNamex->setGeometry(110, 30, 225, 22);
  fileNamex->setText(fd->pathx);
  chFilex = new QPushButton("...", this);
  chFilex->setGeometry(345, 29, 25, 25);
  connect(chFilex, SIGNAL(clicked()), SLOT(slotFilex()));
  l2 = new QLabel(i18n("Function"), this);
  l2->setGeometry(20, 70, 90, 22);
  funcx = new QLineEdit(this);
  funcx->setGeometry(110, 70, 225, 22);
  funcx->setText(fd->namex);
  chFuncx = new QPushButton("...", this);
  chFuncx->setGeometry(345, 69, 25, 25);
  connect(chFuncx, SIGNAL(clicked()), SLOT(slotFuncx()));
  l15 = new QLabel(i18n("Parameter"), this);
  l15->setGeometry(20, 110, 90, 22);
  parx = new QPushButton(i18n("Edit"), this);
  parx->setGeometry(110, 109, 80, 25);
  connect(parx, SIGNAL(clicked()), SLOT(slotEditParameterx()));
  chParx = new QPushButton(i18n("Load"), this);
  chParx->setGeometry(200, 109, 80, 25);
  connect(chParx, SIGNAL(clicked()), SLOT(slotGetParameterx()));
  g2 = new QGroupBox("y(t)", this);
  g2->setGeometry(10, 152, 370, 144);
  l3 = new QLabel(i18n("Library"), this);
  l3->setGeometry(20, 178, 90, 22);
  fileNamey = new QLineEdit(this);
  fileNamey->setGeometry(110, 178, 225, 22);
  fileNamey->setText(fd->pathy);
  chFiley = new QPushButton("...", this);
  chFiley->setGeometry(345, 177, 25, 25);
  connect(chFiley, SIGNAL(clicked()), SLOT(slotFiley()));
  l4 = new QLabel(i18n("Function"), this);
  l4->setGeometry(20, 218, 90, 22);
  funcy = new QLineEdit(this);
  funcy->setGeometry(110, 218, 225, 22);
  funcy->setText(fd->namey);
  chFuncy = new QPushButton("...", this);
  chFuncy->setGeometry(345, 217, 25, 25);
  connect(chFuncy, SIGNAL(clicked()), SLOT(slotFuncy()));
  l16 = new QLabel(i18n("Parameter"), this);
  l16->setGeometry(20, 258, 90, 22);
  pary = new QPushButton(i18n("Edit"), this);
  pary->setGeometry(110, 257, 80, 25);
  connect(pary, SIGNAL(clicked()), SLOT(slotEditParametery()));
  chPary = new QPushButton(i18n("Load"), this);
  chPary->setGeometry(200, 257, 80, 25);
  connect(chPary, SIGNAL(clicked()), SLOT(slotGetParametery()));
  g3 = new QGroupBox(this);
  g3->setGeometry(10, 304, 370, 176);
  l5 = new QLabel("t", this);
  l5->setGeometry(20, 322, 8, 22);
  l6 = new QLabel("min", this);
  l6->setGeometry(28, 326, 28, 22);
  etMin = new QLineEdit(this);
  etMin->setGeometry(110, 322, 50, 22);
  etMin->setText(s.setNum(fd->tmin));
  l7 = new QLabel("t", this);
  l7->setGeometry(230, 322, 8, 22);
  l8 = new QLabel("max", this);
  l8->setGeometry(238, 326, 28, 22);
  etMax = new QLineEdit(this);
  etMax->setGeometry(320, 322, 50, 22);
  etMax->setText(s.setNum(fd->tmax));
  l9 = new QLabel("D", this);
  l9->setGeometry(20, 363, 10, 22);
  QFont f("Symbol");
  l9->setFont(f);
  l10 = new QLabel("t", this);
  l10->setGeometry(30, 362, 54, 22);
  etStep = new QLineEdit(this);
  etStep->setGeometry(110, 362, 50, 22);
  etStep->setText(s.setNum(fd->dt));
  l11 = new QLabel(i18n("x normalization"), this);
  l11->setGeometry(20, 402, 90, 22);
  efx = new QLineEdit(this);
  efx->setGeometry(110, 402, 50, 22);
  efx->setText(s.setNum(fd->fx));
  l12 = new QLabel(i18n("y normalization"), this);
  l12->setGeometry(230, 402, 90, 22);
  efy = new QLineEdit(this);
  efy->setGeometry(320, 402, 50, 22);
  efy->setText(s.setNum(fd->fy));
  l13 = new QLabel(i18n("Symbol"), this);
  l13->setGeometry(20, 444, 90, 22);
  symb = new QSpinBox(-17, 9, 1, this);
  symb->setGeometry(110, 444, 50, 22);
  symb->setValue(fd->symb);
  chSymb = new QPushButton("...", this);
  chSymb->setGeometry(174, 443, 25, 25);
  connect(chSymb, SIGNAL(clicked()), SLOT(slotSymbol()));
  l14 = new QLabel(i18n("Color"), this);
  l14->setGeometry(230, 444, 86, 22);
  colData = new KColorButton(this, 0);
  colData->setGeometry(320, 443, 50, 25);
  colData->setColor(fd->color);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(10, 490, 85, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  apply = new QPushButton(i18n("Apply"), this);
  apply->setGeometry(105, 490, 85, 25);
  connect(apply, SIGNAL(clicked()), SLOT(slotApply()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(200, 490, 85, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(295, 490, 85, 25);
  connect(help, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

ParFuncDlg::~ParFuncDlg()
{
  delete fdt;
  delete l1;
  delete l2;
  delete l3;
  delete l4;
  delete l5;
  delete l6;
  delete l7;
  delete l8;
  delete l9;
  delete l10;
  delete l11;
  delete l12;
  delete l13;
  delete l14;
  delete g1;
  delete g2;
  delete g3;
  delete efx;
  delete efy;
  delete fileNamex;
  delete funcx;
  delete fileNamey;
  delete funcy;
  delete etMin;
  delete etMax;
  delete etStep;
  delete symb;
  delete colData;
  delete chFilex;
  delete chFuncx;
  delete chParx;
  delete parx;
  delete chFiley;
  delete chFuncy;
  delete chPary;
  delete pary;
  delete chSymb;
  delete apply;
  delete ok;
  delete cancel;
  delete help;
}

void ParFuncDlg::getFile(QLineEdit *e)
{
  QString s = KFileDialog::getOpenFileName(0, "*.so");
  if (!s.isEmpty())
    e->setText(s);
}

void ParFuncDlg::slotFilex()
{
  getFile(fileNamex);
}

void ParFuncDlg::slotFiley()
{
  getFile(fileNamey);
}

void ParFuncDlg::getFunc(QLineEdit *name, QLineEdit *func)
{
  char path[260];
  FILE *f;

  QFileInfo fi(name->text());
  sprintf(path, "%s/%s.def", (const char*) fi.dirPath(true),
          (const char*) fi.baseName());
  if ((f = fopen(path, "r"))) {
    ChooseFuncDlg dlg(this, f, func);
    dlg.exec();
  } else
    KMsgBox::message(this, i18n("Error"),
                     i18n("while trying to open module definition file"), 2);
}

void ParFuncDlg::slotFuncx()
{
  getFunc(fileNamex, funcx);
}

void ParFuncDlg::slotFuncy()
{
  getFunc(fileNamey, funcy);
}

void ParFuncDlg::slotGetParameterx()
{
  getPar(this, fdt->px);
}

void ParFuncDlg::slotGetParametery()
{
  getPar(this, fdt->py);
}

void ParFuncDlg::slotEditParameterx()
{
  EditParameterDlg dlg(this, fdt->px);
  dlg.exec();
}

void ParFuncDlg::slotEditParametery()
{
  EditParameterDlg dlg(this, fdt->py);
  dlg.exec();
}

void ParFuncDlg::slotSymbol()
{
  SymbolDlg dlg(this, symb);
  dlg.exec();
}

void ParFuncDlg::getValues(bool ok)
{
  char msg[256], *error;

  if (!getVal(this, efx, &fdt->fx, i18n("invalid fx!")))
    return;
  if (!getVal(this, efy, &fdt->fy, i18n("invalid fy!")))
    return;
  if (!getVal(this, etMin, &fdt->tmin, i18n("invalid tmin!")))
    return;
  if (!getVal(this, etMax, &fdt->tmax, i18n("invalid tmax!")))
    return;
  if (!getVal(this, etStep, &fdt->dt, i18n("invalid dt!")))
    return;
  if (strcmp(fd->pathx, fileNamex->text())) {
    strcpy(fdt->pathx, fileNamex->text());
    if (fdt->pathx[0]) {
      if (!(fdt->hmodx = dlopen(fdt->pathx, RTLD_NOW))) {
        sprintf(msg, i18n("while trying to open\nfile %s:\n%s"),
                fdt->pathx, dlerror());
        KMsgBox::message(this, i18n("Error"), msg, 2);
        return;
      }
    } else
      fdt->hmodx = 0;
  }
  if (strcmp(fd->namex, funcx->text()) || fdt->hmodx) {
    strcpy(fdt->namex, funcx->text());
    if (fdt->namex[0]) {
      if (!fdt->hmodx) {
        KMsgBox::message(this, i18n("Error"),
                         i18n("while trying to open library"), 2);
        return;
      }
      fdt->fktx = (double (*)(double, double *)) dlsym(fdt->hmodx, fdt->namex);
      if ((error = (char *) dlerror()) != 0) {
        sprintf(msg, i18n("while trying to determine the address of\nfunction %s:\n%s"),
                fdt->namex, error);
        KMsgBox::message(this, i18n("Error"), msg, 2);
      }
    } else
      fdt->fktx = 0;
  }
  if (strcmp(fd->pathy, fileNamey->text())) {
    strcpy(fdt->pathy, fileNamey->text());
    if (fdt->pathy[0]) {
      if (!(fdt->hmody = dlopen(fdt->pathy, RTLD_NOW))) {
        sprintf(msg, i18n("while trying to open\nfile %s:\n%s"),
                fdt->pathy, dlerror());
        KMsgBox::message(this, i18n("Error"), msg, 2);
        return;
      }
    } else
      fdt->hmody = 0;
  }
  if (strcmp(fd->namey, funcy->text()) || fdt->hmody) {
    strcpy(fdt->namey, funcy->text());
    if (fdt->namey[0]) {
      if (!fdt->hmody) {
        KMsgBox::message(this, i18n("Error"),
                         i18n("while trying to open library"), 2);
        return;
      }
      fdt->fkty = (double (*)(double, double *)) dlsym(fdt->hmody, fdt->namey);
      if ((error = (char *) dlerror()) != 0) {
        sprintf(msg, i18n("while trying to determine the address of\nfunction %s:\n%s"),
                fdt->namey, error);
        KMsgBox::message(this, i18n("Error"), msg, 2);
      }
    } else
      fdt->fkty = 0;
  }
  if (fd->hmodx && (fd->hmodx != fdt->hmodx))
    dlclose(fd->hmodx);
  if (fd->hmody && (fd->hmody != fdt->hmody))
    dlclose(fd->hmody);
  *fd = *fdt;
  fd->color = colData->color().rgb();
  fd->symb = symb->value();
  kp->changed = true;
  kp->updWin();
  if (ok)
    accept();
}

void ParFuncDlg::slotApply()
{
  getValues(false);
}

void ParFuncDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-parfunc");
}

void ParFuncDlg::slotOK()
{
  getValues(true);
}

