/****************************************************************/
/*                                                              */
/*      funcdlg.h                                               */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#ifndef FUNCDLG_H
#define FUNCDLG_H

#include "kplots.h"
#include <qdialog.h>
#include <stdio.h>

class KPl;
class KColorButton;
class QLineEdit;
class QListBox;
class QLabel;
class QGroupBox;
class QSpinBox;

class ChooseFuncDlg : public QDialog {
  Q_OBJECT
  public:
    ChooseFuncDlg(QWidget *parent, FILE *f, QLineEdit *func0);
    ~ChooseFuncDlg();
  public slots:
    void slotList(int i);
    void slotOK();
  private:
    QLineEdit *func;
    QListBox *funcList;
    QPushButton *ok, *cancel;
};

class EditParameterDlg : public QDialog {
  Q_OBJECT
  public:
    EditParameterDlg(QWidget *parent, double *p0);
    ~EditParameterDlg();
  public slots:
    void slotOK();
  private:
    double *p;
    QLabel *lp[20], *ln[20];
    QLineEdit *par[20];
    QPushButton *ok, *cancel;
};

class FuncDlg : public QDialog {
  Q_OBJECT
  public:
    FuncDlg(QWidget *parent, KPl *kpa, PFUNDATA fd);
    ~FuncDlg();
  private slots:
    void slotFile();
    void slotFunc();
    void slotGetParameter();
    void slotEditParameter();
    void slotSymbol();
    void slotApply();
    void slotHelp();
    void slotOK();
  private:
    void getValues(bool accept);
    KPl *kp;
    PFUNDATA fd, fdt;
    QLabel *l1, *l2, *l3, *l4, *l5, *l6, *l7, *l8, *l9, *l10, *l11, *l12,
           *l13;
    QLineEdit *fileName, *func, *exMin, *exMax, *exStep, *efx, *efy;
    QGroupBox *g1, *g2;
    QSpinBox *symb;
    KColorButton *colData;
    QPushButton *chFile, *chFunc, *chPar, *par, *chSymb, *apply,
                *ok, *cancel, *help;
};

class ParFuncDlg : public QDialog {
  Q_OBJECT
  public:
    ParFuncDlg(QWidget *parent, KPl *kpa, PPARFUNDATA fd);
    ~ParFuncDlg();
  private slots:
    void slotFilex();
    void slotFuncx();
    void slotGetParameterx();
    void slotEditParameterx();
    void slotFiley();
    void slotFuncy();
    void slotGetParametery();
    void slotEditParametery();
    void slotSymbol();
    void slotApply();
    void slotHelp();
    void slotOK();
  private:
    void getFile(QLineEdit *e);
    void getFunc(QLineEdit *name, QLineEdit *func);
    void getValues(bool accept);
    KPl *kp;
    PPARFUNDATA fd, fdt;
    QLabel *l1, *l2, *l3, *l4, *l5, *l6, *l7, *l8, *l9, *l10,
           *l11, *l12, *l13, *l14, *l15, *l16;
    QLineEdit *fileNamex, *funcx, *fileNamey, *funcy, *etMin, *etMax,
              *etStep, *efx, *efy;
    QGroupBox *g1, *g2, *g3;
    QSpinBox *symb;
    KColorButton *colData;
    QPushButton *chFilex, *chFuncx, *chParx, *parx,
                *chFiley, *chFuncy, *chPary, *pary,
                *chSymb, *apply, *ok, *cancel, *help;
};

#endif

