/****************************************************************/
/*                                                              */
/*      kpl.h                                                   */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#ifndef KPL_H
#define KPL_H

#include <ktmainwindow.h>
#include <kgraph.h>
#include "kplots.h"
#include "basicdlg.h"
#include "autodlg.h"

class KDNDDropZone;
class ItemDlg;
class FrameDlg;
class ArrayDlg;
class FuncDlg;
class ParFuncDlg;
class LegendDlg;
class KPlWidget;
class KPlView;

#define IDM_NEW                     1110
#define IDM_OPENPLO                 1120
#define IDM_OPENDAT                 1130
#define IDM_SAVE                    1140
#define IDM_SAVEAS                  1150
#define IDM_PLOT                    1160
#define IDM_PS                      1170
#define IDM_PRINT                   1180
#define IDM_NEWWINDOW               1185
#define IDM_CLOSE                   1189
#define IDM_EXIT                    1190

#define IDM_ITEMS                   1200

#define IDM_TOOLBAR                 1310
#define IDM_STATUSBAR               1320
#define IDM_AUTOPLOT                1330
#define IDM_SAVEATEND               1340
#define IDM_BASICSETTINGS           1350
#define IDM_AUTOSETTINGS            1360
#define IDM_SAVEOPTIONS             1370

#define IDM_ZOOM                    1410
#define IDM_REFRESH                 1420
#define IDM_AUTOREFRESH             1430

#define IDD_STATUSTEXT              2901
#define IDD_STATUSTEXT2             2902

#define SCROLLBAR_SIZE 16

class KPl : public KTMainWindow, public KGraph {
  Q_OBJECT
  friend ItemDlg;
  friend FrameDlg;
  friend ArrayDlg;
  friend FuncDlg;
  friend ParFuncDlg;
  friend LegendDlg;
  friend KPlWidget;
  friend KPlView;
  public:
    KPl();
    ~KPl();
    void getFile(QString path);
  private slots:
    void slotNew();
    void slotItems();
    void slotOpenPlot();
    void slotOpenData();
    void slotSave();
    void slotSaveAs();
    void slotPrint();
    void slotClose();
    void slotExit();
    void slotDropEvent(KDNDDropZone *);
    void slotScrollLeft();
    void slotScrollRight();
    void slotScrollUp();
    void slotScrollDown();
    void slotScrollPageUp();
    void slotScrollPageDown();
    void slotScrollPageLeft();
    void slotScrollPageRight();
    void slotScrollHomeLeft();
    void slotScrollHomeDown();
    void slotScrollEndRight();
    void slotScrollEndUp();
    void slotHelp();
    void commandCallback(int id);
  protected:
    void timerEvent(QTimerEvent *e);
    void resizeEvent(QResizeEvent *);
  private:
    void freeX(double ***x);
    void freeXCond(PARRAYDATA ad);
    void freeItd(PITEMDATA iti);
    void freeAllItd();
    int readFile(QString filename, double ***x, int *ncols, int *nrows);
    void iniStru(PITEMDATA itd, int ityp);
    int autoStruc(PFRAMEDATA fd, PARRAYDATA ad);
    void updWin();
    void getFun(QString path, QString name, void **hmod,
                double (**fkt)(double, double *));
    int readPlo(FILE *f);
    void writePlo(FILE *f);
    FILE *openFile(QString *fileBuf, QString fileName);
    void open(int iType);
    bool save(int id);
    void drawFrame(PFRAMEDATA fd);
    void drawArray(PARRAYDATA ad);
    void drawFun(PFUNDATA fd);
    void drawParFun(PPARFUNDATA pfd);
    void drawLegend(PLEGENDDATA ld);
    void doDraw();
    void psFrame(PFRAMEDATA fd);
    void psArray(PARRAYDATA ad);
    void psFun(PFUNDATA fd);
    void psParFun(PPARFUNDATA pfd);
    void psLegend(PLEGENDDATA ld);
    void doPS(FILE *f);
    void updStatus(int x, int y);
    void paintWidget();
    void saveOptions();
    void calibrateScrollBars();
    void shrink();
    bool queryExit();
    bool querySave();
    static const int iSize[6];
    static const char itemType[6][11];
    static QList<KPl> windowList;
    bool changed, timerActive;
    int plot, nItems, fType, nyLeg, xPict, yPict, scrx, scry,
        tRefresh, idTimer;
    double fxn, fyn, fZoom;
    QString currentPath, currentDir;
    basicStruct bas;
    autoStruct aut;
    PITEMDATA itd;
    KPlWidget *plWidget;
    QPopupMenu *filePopup, *optionsPopup, *displayPopup;
};

#endif
