/****************************************************************/
/*                                                              */
/*      legenddlg.cpp                                           */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "legenddlg.h"
#include "kpl.h"
#include "edittextdlg.h"
#include "symboldlg.h"
#include "utils.h"
#include <kapp.h>
#include <kcolorbtn.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qgroupbox.h>

LegendDlg::LegendDlg(QWidget *parent, KPl *kpa, PLEGENDDATA ld0) :
 QDialog(parent, 0, TRUE)
{
  QString s;

  kp = kpa;
  ld = ld0;
  setCaption(i18n("Legend"));
  g1 = new QGroupBox(i18n("Data"), this);
  g1->setGeometry(10, 6, 370, 94);
  l1 = new QLabel(i18n("Symbol"), this);
  l1->setGeometry(20, 34, 60, 22);
  symb = new QSpinBox(-17, 9, 1, this);
  symb->setGeometry(90, 34, 50, 22);
  symb->setValue(ld->symb);
  chSymb = new QPushButton("...", this);
  chSymb->setGeometry(150, 33, 25, 25);
  connect(chSymb, SIGNAL(clicked()), SLOT(slotSymbol()));
  l2 = new QLabel(i18n("Color"), this);
  l2->setGeometry(228, 32, 54, 25);
  colSymb = new KColorButton(this, 0);
  colSymb->setGeometry(282, 32, 50, 25);
  colSymb->setColor(ld->colsymb);
  l3 = new QLabel("x", this);
  l3->setGeometry(20, 64, 58, 22);
  ex0 = new QLineEdit(this);
  ex0->setGeometry(90, 64, 50, 22);
  ex0->setText(s.setNum(ld->x, 'f', 2));
  l4 = new QLabel("cm", this);
  l4->setGeometry(148, 64, 26, 22);
  l5 = new QLabel("y", this);
  l5->setGeometry(228, 64, 58, 22);
  ey0 = new QLineEdit(this);
  ey0->setGeometry(282, 64, 50, 22);
  ey0->setText(s.setNum(ld->y, 'f', 2));
  l6 = new QLabel("cm", this);
  l6->setGeometry(338, 64, 26, 22);
  g2 = new QGroupBox(i18n("Label"), this);
  g2->setGeometry(10, 106, 370, 134);
  l7 = new QLabel(i18n("Text"), this);
  l7->setGeometry(20, 132, 58, 22);
  eText = new QLineEdit(this);
  eText->setGeometry(90, 132, 242, 22);
  eText->setText(ld->s);
  chText = new QPushButton("...", this);
  chText->setGeometry(346, 132, 25, 25);
  connect(chText, SIGNAL(clicked()), SLOT(slotEditText()));
  l8 = new QLabel(i18n("Size"), this);
  l8->setGeometry(20, 168, 70, 22);
  eRelSize = new QSpinBox(10, 800, 1, this);
  eRelSize->setGeometry(90, 168, 50, 22);
  eRelSize->setValue((int) (100 * ld->relsiz + 0.5));
  l9 = new QLabel("%", this);
  l9->setGeometry(142, 168, 30, 22);
  l10 = new QLabel(i18n("Color"), this);
  l10->setGeometry(228, 167, 54, 25);
  colText = new KColorButton(this, 0);
  colText->setGeometry(282, 167, 50, 25);
  colText->setColor(ld->coltext);
  l11 = new QLabel("x", this);
  l11->setGeometry(20, 204, 8, 22);
  l12 = new QLabel("off", this);
  l12->setGeometry(28, 208, 50, 22);
  exl = new QLineEdit(this);
  exl->setGeometry(90, 204, 50, 22);
  exl->setText(s.setNum(ld->xtext, 'f', 2));
  l13 = new QLabel("cm", this);
  l13->setGeometry(142, 204, 26, 22);
  l14 = new QLabel("y", this);
  l14->setGeometry(228, 204, 8, 22);
  l15 = new QLabel("off", this);
  l15->setGeometry(236, 208, 46, 22);
  eyl = new QLineEdit(this);
  eyl->setGeometry(282, 204, 50, 22);
  eyl->setText(s.setNum(ld->ytext, 'f', 2));
  l16 = new QLabel("cm", this);
  l16->setGeometry(338, 204, 26, 22);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(10, 252, 85, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  apply = new QPushButton(i18n("Apply"), this);
  apply->setGeometry(105, 252, 85, 25);
  connect(apply, SIGNAL(clicked()), SLOT(slotApply()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(200, 252, 85, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(295, 252, 85, 25);
  connect(help, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

LegendDlg::~LegendDlg()
{
  delete l1;
  delete l2;
  delete l3;
  delete l4;
  delete l5;
  delete l6;
  delete l7;
  delete l8;
  delete l9;
  delete l10;
  delete l11;
  delete l12;
  delete l13;
  delete l14;
  delete l15;
  delete l16;
  delete g1;
  delete g2;
  delete ex0;
  delete ey0;
  delete exl;
  delete eyl;
  delete eText;
  delete symb;
  delete eRelSize;
  delete colSymb;
  delete colText;
  delete chSymb;
  delete chText;
  delete apply;
  delete ok;
  delete cancel;
  delete help;
}

void LegendDlg::slotSymbol()
{
  SymbolDlg dlg(this, symb);
  dlg.exec();
}

void LegendDlg::slotEditText()
{
  EditTextDlg dlg(this, eText);
  dlg.exec();
}

void LegendDlg::getValues(bool ok)
{
  double x, y, xtext, ytext;

  if (!getVal(this, ex0, &x, i18n("invalid x position!")))
    return;
  if (!getVal(this, ey0, &y, i18n("invalid y position!")))
    return;
  if (!getVal(this, exl, &xtext, i18n("invalid x position for text!")))
    return;
  if (!getVal(this, eyl, &ytext, i18n("invalid y position for text!")))
    return;
  ld->x = x;
  ld->y = y;
  ld->xtext = xtext;
  ld->ytext = ytext;
  ld->colsymb = colSymb->color().rgb();
  ld->coltext = colText->color().rgb();
  ld->symb = symb->value();
  ld->relsiz = 0.01 * eRelSize->value();
  strcpy(ld->s, eText->text());
  kp->changed = true;
  kp->updWin();
  if (ok)
    accept();
}

void LegendDlg::slotApply()
{
  getValues(false);
}

void LegendDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-legend");
}

void LegendDlg::slotOK()
{
  getValues(true);
}

