/****************************************************************/
/*                                                              */
/*      pslib.h                                                 */
/*                                                              */
/*      Graphic functions for PostScript output                 */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#ifndef PSLIB_H
#define PSLIB_H

#if defined(_Windows) || defined(__WIN32__)
  #include <windows.h>
#endif

#ifdef __GNUC__
  #ifdef __WIN32__
    #define _api WINAPI
  #else
    #define _api
    #define FAR
  #endif
#else
  #define _api FAR WINAPI
#endif

#ifdef __cplusplus
extern "C" {
#endif

extern void psInit(FILE *f);
extern void psFormat(double x, double y);
extern void psWindow(double x0, double xl, double y0, double yl);
extern double psRelSize(double r);
extern void psScale(double xl, double xr, double yb, double yt);
extern void psSetDig(int nx, int ny);
extern void psSetColFrame(long icol);
extern void psSetColGrid(long icol);
extern void psSetColData(long icol);
extern void psSetCol(long icol1, long icol2, long icol3);
extern void psFrame(void);
extern void psRaster(double xtic, double ytic, int intx, int inty, int mode);
extern void psText(char *str, double x, double y, int iorg);
extern void psLetX(char *str, int iex);
extern void psLetY(char *str, int iey);
extern void psLetH(char *str);
extern void psSymbol(int is);
extern void psPlArray(double *x, double *y, double fx, double fy, int n);
extern void psPlError(double *x, double *y, double *e, double fx, double fy, int n);
extern void psPlFun(double xfmin, double xfmax, double dx, double fx, double fy,
                    double (_api *func)(double, double FAR *), double FAR *p);
extern void psPlParFun(double tmin, double tmax, double dt, double fx, double fy,
                       double (_api *funcx)(double, double FAR *), double FAR *px,
                       double (_api *funcy)(double, double FAR *), double FAR *py);
extern void psExit(void);

#ifdef __cplusplus
}
#endif

#endif
